@@ -20,12 +20,20 @@
 #ifndef __CAIRO_DOCK_CONTAINER__
 #define  __CAIRO_DOCK_CONTAINER__
 
+#include "gldi-config.h"
 #include <glib.h>
+#ifdef HAVE_GLX
+#include <GL/glx.h>  // GLXContext
+#endif
 
 #include "cairo-dock-struct.h"
 #include "cairo-dock-manager.h"
-G_BEGIN_DECLS
 
+#if (GTK_MAJOR_VERSION < 3)
+#include <gdk/gdkkeysyms.h> // GDK_Shift_[LR] for GTK2
+#endif
+
+G_BEGIN_DECLS
 
 /**
 *@file cairo-dock-container.h This class defines the Containers, that are classic or hardware accelerated animated windows, and exposes common functions, such as redrawing a part of a container or popping a menu on a container.
@@ -34,37 +42,36 @@ G_BEGIN_DECLS
 *
 * Docks, Desklets, Dialogs, and Flying-containers all derive from Containers.
 *
-* If you write a new type of container, you must call \ref cairo_dock_init_container when you create it and \ref cairo_dock_finish_container when you destroy it.
 */
 
-typedef struct _CairoContainersParam CairoContainersParam;
-typedef struct _CairoContainersManager CairoContainersManager;
+// manager
+typedef struct _GldiContainersParam GldiContainersParam;
+typedef struct _GldiContainerAttr GldiContainerAttr;
 
 #ifndef _MANAGER_DEF_
-extern CairoContainersParam myContainersParam;
-extern CairoContainersManager myContainersMgr;
+extern GldiContainersParam myContainersParam;
+extern GldiManager myContainersMgr;
+extern GldiObjectManager myContainerObjectMgr;
 #endif
 
 #define CD_DOUBLE_CLICK_DELAY 250  // ms
 
-
 // params
-struct _CairoContainersParam{
-	gboolean bUseFakeTransparency;
+struct _GldiContainersParam{
+	//gboolean bUseFakeTransparency;
 	gint iGLAnimationDeltaT;
 	gint iCairoAnimationDeltaT;
 	};
 
-// manager
-struct _CairoContainersManager {
-	GldiManager mgr;
-	} ;
+struct _GldiContainerAttr {
+	gboolean bNoOpengl;
+};
 
 /// signals
 typedef enum {
-	/// notification called when the menu is being built on a container. data : {Icon, CairoContainer, GtkMenu, gboolean*}
+	/// notification called when the menu is being built on a container. data : {Icon, GldiContainer, GtkMenu, gboolean*}
 	NOTIFICATION_BUILD_CONTAINER_MENU = NB_NOTIFICATIONS_OBJECT,
-	/// notification called when the menu is being built on an icon (possibly NULL). data : {Icon, CairoContainer, GtkMenu}
+	/// notification called when the menu is being built on an icon (possibly NULL). data : {Icon, GldiContainer, GtkMenu}
 	NOTIFICATION_BUILD_ICON_MENU,
 	/// notification called when use clicks on an icon data : {Icon, CairoDock, int}
 	NOTIFICATION_CLICK_ICON,
@@ -91,7 +98,7 @@ typedef enum {
 	/// notification called when a container is rendered.
 	NOTIFICATION_RENDER,
 	NB_NOTIFICATIONS_CONTAINER
-	} CairoContainerNotifications;
+	} GldiContainerNotifications;
 
 
 // factory
@@ -101,28 +108,19 @@ typedef enum {
 	CAIRO_DOCK_HORIZONTAL
 	} CairoDockTypeHorizontality;
 
-/// Types of available containers.
-typedef enum {
-	CAIRO_DOCK_TYPE_DOCK = 0,
-	CAIRO_DOCK_TYPE_DESKLET,
-	CAIRO_DOCK_TYPE_DIALOG,
-	CAIRO_DOCK_TYPE_FLYING_CONTAINER,
-	CAIRO_DOCK_NB_CONTAINER_TYPES
-	} CairoDockTypeContainer;
-
-struct _CairoContainerInterface {
-	gboolean (*animation_loop) (CairoContainer *pContainer);
-	void (*setup_menu) (CairoContainer *pContainer, Icon *pIcon, GtkWidget *pMenu);
+struct _GldiContainerInterface {
+	gboolean (*animation_loop) (GldiContainer *pContainer);
+	void (*setup_menu) (GldiContainer *pContainer, Icon *pIcon, GtkWidget *pMenu);
+	void (*detach_icon) (GldiContainer *pContainer, Icon *pIcon);
+	void (*insert_icon) (GldiContainer *pContainer, Icon *pIcon, gboolean bAnimateIcon);
 	};
 
 /// Definition of a Container, whom derive Dock, Desklet, Dialog and FlyingContainer. 
-struct _CairoContainer {
+struct _GldiContainer {
 	/// object.
 	GldiObject object;
 	/// External data.
 	gpointer pDataSlot[CAIRO_DOCK_NB_DATA_SLOT];
-	/// type of container.
-	CairoDockTypeContainer iType;
 	/// window of the container.
 	GtkWidget *pWidget;
 	/// size of the container.
@@ -147,60 +145,53 @@ struct _CairoContainer {
 	gdouble fRatio;
 	/// TRUE if the container has a reflection power.
 	gboolean bUseReflect;
+	#ifdef HAVE_GLX
 	/// OpenGL context.
 	GLXContext glContext;
+	#else
+	gpointer unused;  // to keep ABI compatibility for plug-ins
+	#endif
 	/// whether the GL context is an ortho or a perspective view.
 	gboolean bPerspectiveView;
 	/// TRUE if a slow animation is running.
 	gboolean bKeepSlowAnimation;
 	/// counter for the animation loop.
 	gint iAnimationStep;
-	CairoContainerInterface iface;
+	GldiContainerInterface iface;
 	
 	gboolean bIgnoreNextReleaseEvent;
 	gpointer reserved[4];
 };
 
-/// Get the Container part of a pointer.
-#define CAIRO_CONTAINER(p) ((CairoContainer *) (p))
 
+/// Definition of the Container backend. It defines some operations that should be, but are not, provided by GTK.
+struct _GldiContainerManagerBackend {
+	void (*reserve_space) (GldiContainer *pContainer, int left, int right, int top, int bottom, int left_start_y, int left_end_y, int right_start_y, int right_end_y, int top_start_x, int top_end_x, int bottom_start_x, int bottom_end_x);
+	int (*get_current_desktop_index) (GldiContainer *pContainer);
+	void (*move) (GldiContainer *pContainer, int iNumDesktop, int iAbsolutePositionX, int iAbsolutePositionY);
+	gboolean (*is_active) (GldiContainer *pContainer);
+	void (*present) (GldiContainer *pContainer);
+};
+
+
+/// Get the Container part of a pointer.
+#define CAIRO_CONTAINER(p) ((GldiContainer *) (p))
+
+/** Say if an object is a Container.
+*@param obj the object.
+*@return TRUE if the object is a Container.
+*/
+#define CAIRO_DOCK_IS_CONTAINER(obj) gldi_object_is_manager_child (GLDI_OBJECT(obj), &myContainerObjectMgr)
 
   /////////////
  // WINDOW //
 ///////////
 
-#define gldi_container_new_full(Type, mgr, type, bOpenGLWindow) \
-__extension__ ({ \
-	Type *obj = gldi_object_new (Type, mgr); \
-	CAIRO_CONTAINER(obj)->iType = type; \
-	cairo_dock_init_container_full (CAIRO_CONTAINER (obj), bOpenGLWindow); \
-	obj; })
-
-#define gldi_container_new(Type, iType, mgr) gldi_container_new_full (Type, iType, mgr, TRUE)
 
 void cairo_dock_set_containers_non_sticky (void);
 
 void cairo_dock_disable_containers_opacity (void);
 
-GtkWidget *cairo_dock_init_container_full (CairoContainer *pContainer, gboolean bOpenGLWindow);
-
-/** Initialize a Container : create a GTK window with transparency and OpenGL support. To be called when you create a new container.
-*@param pContainer a Container.
-*@return the newly allocated GTK window.
-*/
-#define cairo_dock_init_container(pContainer) cairo_dock_init_container_full (pContainer, TRUE)
-
-/** Same as above, but with no OpenGL support.
-*@param pContainer a Container.
-*/
-#define cairo_dock_init_container_no_opengl(pContainer) cairo_dock_init_container_full (pContainer, FALSE)
-
-/** Finish a Container. To be called before you free it.
-*@param pContainer a Container.
-*/
-void cairo_dock_finish_container (CairoContainer *pContainer);
-
-
 #if (GTK_MAJOR_VERSION < 3 && GTK_MINOR_VERSION < 14)
 #define gldi_container_get_gdk_window(pContainer) (pContainer)->pWidget->window
 #else
@@ -247,39 +238,75 @@ void cairo_dock_finish_container (CairoContainer *pContainer);
 		gdk_window_get_device_position (gldi_container_get_gdk_window (pContainer), pDevice, &pContainer->iMouseY, &pContainer->iMouseX, NULL); } while (0)
 #endif
 
-gboolean cairo_dock_emit_signal_on_container (CairoContainer *pContainer, const gchar *cSignal);
-gboolean cairo_dock_emit_leave_signal (CairoContainer *pContainer);
-gboolean cairo_dock_emit_enter_signal (CairoContainer *pContainer);
+
+/** Reserve a space on the screen for a Container; other windows won't overlap this space when maximised.
+*@param pContainer the container
+*@param left 
+*@param right 
+*@param top 
+*@param bottom 
+*@param left_start_y 
+*@param left_end_y 
+*@param right_start_y 
+*@param right_end_y 
+*@param top_start_x 
+*@param top_end_x 
+*@param bottom_start_x 
+*@param bottom_end_x 
+*/
+void gldi_container_reserve_space (GldiContainer *pContainer, int left, int right, int top, int bottom, int left_start_y, int left_end_y, int right_start_y, int right_end_y, int top_start_x, int top_end_x, int bottom_start_x, int bottom_end_x);
+
+/** Get the desktop and viewports a Container is placed on.
+*@param pContainer the container
+*@return an index representing the desktop and viewports.
+*/
+int gldi_container_get_current_desktop_index (GldiContainer *pContainer);
+
+/** Move a Container to a given desktop, viewport, and position (similar to gtk_window_move except that the position is defined on the whole desktop (made of all viewports); it's only useful if the Container is sticky).
+*@param pContainer the container
+*@param iNumDesktop desktop number
+*@param iAbsolutePositionX horizontal position on the virtual screen
+*@param iAbsolutePositionY vertical position on the virtual screen
+*/
+void gldi_container_move (GldiContainer *pContainer, int iNumDesktop, int iAbsolutePositionX, int iAbsolutePositionY);
+
+/** Tell if a Container is the current active window (similar to gtk_window_is_active but actually works).
+*@param pContainer the container
+*@return TRUE if the Container is the current active window.
+*/
+gboolean gldi_container_is_active (GldiContainer *pContainer);
+
+/** Show a Container and make it take the focus  (similar to gtk_window_present, but bypasses the WM focus steal prevention).
+*@param pContainer the container
+*/
+void gldi_container_present (GldiContainer *pContainer);
+
+
+void gldi_container_manager_register_backend (GldiContainerManagerBackend *pBackend);
+
 
 
   ////////////
  // REDRAW //
 ////////////
 
+void cairo_dock_set_default_rgba_visual (GtkWidget *pWidget);
+
 /** Clear and trigger the redraw of a Container.
 *@param pContainer the Container to redraw.
 */
-void cairo_dock_redraw_container (CairoContainer *pContainer);
+void cairo_dock_redraw_container (GldiContainer *pContainer);
 
 /** Clear and trigger the redraw of a part of a container.
 *@param pContainer the Container to redraw.
 *@param pArea the zone to redraw.
 */
-void cairo_dock_redraw_container_area (CairoContainer *pContainer, GdkRectangle *pArea);
+void cairo_dock_redraw_container_area (GldiContainer *pContainer, GdkRectangle *pArea);
 
 /** Clear and trigger the redraw of an Icon. The drawing is not done immediately, but when the expose event is received.
 *@param icon l'icone a retracer.
-*@param pContainer le container de l'icone.
 */
-void cairo_dock_redraw_icon (Icon *icon, CairoContainer *pContainer);
-
-
-
-/** Search for the Container of a given Icon (dock or desklet in the case of an applet).
-* @param icon the icon.
-* @return the container contening this icon, or NULL if the icon is nowhere.
-*/
-CairoContainer *cairo_dock_search_container_from_icon (Icon *icon);
+void cairo_dock_redraw_icon (Icon *icon);
 
 
 void cairo_dock_allow_widget_to_receive_data (GtkWidget *pWidget, GCallback pCallBack, gpointer data);
@@ -291,13 +318,7 @@ void cairo_dock_allow_widget_to_receive_data (GtkWidget *pWidget, GCallback pCal
 */
 #define gldi_container_enable_drop(pContainer, pCallBack, data) cairo_dock_allow_widget_to_receive_data (pContainer->pWidget, pCallBack, data)
 
-void gldi_container_disable_drop (CairoContainer *pContainer);
-
-/** Say if a string is an adress (file://xxx, http://xxx, ftp://xxx, etc).
-* @param cString a string.
-* @return TRUE if it's an address.
-*/
-gboolean cairo_dock_string_is_adress (const gchar *cString);
+void gldi_container_disable_drop (GldiContainer *pContainer);
 
 /** Notify everybody that a drop has just occured.
 * @param cReceivedData the dropped data.
@@ -305,51 +326,19 @@ gboolean cairo_dock_string_is_adress (const gchar *cString);
 * @param fOrder the order of the icon if the drop occured on it, or LAST_ORDER if the drop occured between 2 icons.
 * @param pContainer the container of the icon
 */
-void cairo_dock_notify_drop_data (gchar *cReceivedData, Icon *pPointedIcon, double fOrder, CairoContainer *pContainer);
+void gldi_container_notify_drop_data (GldiContainer *pContainer, gchar *cReceivedData, Icon *pPointedIcon, double fOrder);
 
 
-  //////////
- // MENU //
-//////////
-
-/** Pop-up a menu on an icon. The menu is placed so that it touches the icon, without overlapping it. If the icon is NULL, it will be placed it at the mouse's position. In the case of a dock, it prevents this one from shrinking down.
-*@param menu the menu.
-*@param pIcon the icon, or NULL.
-*@param pContainer the container that was clicked.
-*/
-void cairo_dock_popup_menu_on_icon (GtkWidget *menu, Icon *pIcon, CairoContainer *pContainer);
-
-/** Pop-up a menu on a container. In the case of a dock, it prevents this one from shrinking down.
-*@param menu the menu.
-*@param pContainer the container that was clicked.
-*/
-#define cairo_dock_popup_menu_on_container(menu, pContainer) cairo_dock_popup_menu_on_icon (menu, NULL, pContainer)
-
-/** Add an entry to a given menu.
-*@param cLabel label of the entry
-*@param gtkStock a GTK stock or a path to an image
-*@param pFunction callback
-*@param pMenu the menu to insert the entry in
-*@param pData data to feed the callback with
-* @return the new menu-entry that has been added.
-*/
-GtkWidget *cairo_dock_add_in_menu_with_stock_and_data (const gchar *cLabel, const gchar *gtkStock, GCallback pFunction, GtkWidget *pMenu, gpointer pData);
-
-/** Add sub-menu to a given menu.
-*@param cLabel label of the sub-menu
-*@param pMenu the menu to insert the entry in
-*@param cImage a GTK stock or a path to an image
-* @return the new sub-menu that has been added.
-*/
-GtkWidget *cairo_dock_create_sub_menu (const gchar *cLabel, GtkWidget *pMenu, const gchar *cImage);
-
+gboolean cairo_dock_emit_signal_on_container (GldiContainer *pContainer, const gchar *cSignal);
+gboolean cairo_dock_emit_leave_signal (GldiContainer *pContainer);
+gboolean cairo_dock_emit_enter_signal (GldiContainer *pContainer);
 
 /** Build the main menu of a Container.
 *@param icon the icon that was left-clicked, or NULL if none.
 *@param pContainer the container that was left-clicked.
 *@return the menu.
 */
-GtkWidget *cairo_dock_build_menu (Icon *icon, CairoContainer *pContainer);
+GtkWidget *gldi_container_build_menu (GldiContainer *pContainer, Icon *icon);
 
 #if (GTK_MAJOR_VERSION < 3)
 #define _gtk_hbox_new(m) gtk_hbox_new (FALSE, m)
@@ -360,20 +349,11 @@ GtkWidget *cairo_dock_build_menu (Icon *icon, CairoContainer *pContainer);
 #endif
 
 
-#if ((CAIRO_DOCK_FORCE_ICON_IN_MENUS == 1) && (GTK_MAJOR_VERSION > 2 || GTK_MINOR_VERSION >= 16))
-	#define _gtk_image_menu_item_set_image(pMenuItem, image) do {\
-		gtk_image_menu_item_set_image (pMenuItem, image);\
-		gtk_image_menu_item_set_always_show_image (pMenuItem, TRUE); } while (0)
-#else
-	#define _gtk_image_menu_item_set_image gtk_image_menu_item_set_image
-#endif
-
-
   /////////////////
  // INPUT SHAPE //
 /////////////////
 
-GldiShape *gldi_container_create_input_shape (CairoContainer *pContainer, int x, int y, int w, int h);
+GldiShape *gldi_container_create_input_shape (GldiContainer *pContainer, int x, int y, int w, int h);
 
 #if (GTK_MAJOR_VERSION < 3)
 #define gldi_container_set_input_shape(pContainer, pShape) \
